// serial.cpp : Implementation of the Cserial class
//
/*******************************************************************************
*                           OPEN SOURCE CODE DISCLAIMER                        *
* FILE NAME:          rs4.cpp                                                  *
* DATE:               December 2001                                            *
* PURPOSE:            Control a Leuze RotoScan 4 Laser Unit with a Serial Port *
* ORIGINAL CODE BY:   Juan Andrade Cetto & Albert Checa Puimedon               *
*                     (C) Institut de Robtica Industrial                      *
*                     Universidad Politcnica de Catalunya                     *
* ALGORITHM BASED ON: RS4_protocol_extern-e.doc (1.0) Leuze electronic         *
* CONTRIBUTOR(S):                                                              *
*                                                                              *
* The contents of this file are subject to the Mozilla Public License; you may *
* not use this file except in compliance with the License. You may obtain a    *
* copy of the License at http://www.mozilla.org/MPL/                           *
* Software distributed under the License is distributed on an "AS IS" basis,   *
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for *
* the specific language governing rights and limitations under the License.    *
*                                                                              *
* You may modify this code (or portions thereof) provided a copy of this       *
* disclaimer is included in the modified version.                              *
*******************************************************************************/
	


#include "serial.h"



/*JF: Variables relacionadas con portstream handle*/
portstream_fd pstream;
static char err;
double elapsed_time;


/*JF: Funcion permite realizar la apertura y configuracion del puerto serial*/

portstream_fd openserial(char *portname)
{	 /* Serial communications parameters */
 	 int        speed    = 57600;
	 int        parity   = NO_PARITY;
	 int        bits     = 8;
	 int        stopbits = 1;
	 
	 portstream_fd pstream;
	// peeked_char_avail = FALSE;

	 pstream = setserial(portname, speed, parity, bits, stopbits);
	 return pstream;
}

/*JF: Funcion permite realizar el cierre del puerto serial*/
char closeserial(portstream_fd portstream)
{   if (portstream != NULL) {                         
   		// disable event notification
   		SetCommMask( portstream, 0 ) ;
   	    // drop DTR
   		EscapeCommFunction( portstream, CLRDTR ) ;	 
   		// purge any outstanding reads/writes and close device handle
        PurgeComm( portstream, PURGE_TXABORT | PURGE_RXABORT | 
			                   PURGE_TXCLEAR | PURGE_RXCLEAR );
		// close the device   
		err = CloseHandle( portstream ) ;
        portstream = (HANDLE) -1;                
		if (err < 0)
			return -1;
	 }
     return 0;
}

/*JF: Funcion permite realizar la configuracion del puerto serial*/
portstream_fd setserial(char *portname, int Speed, int Parity, int Bits, int StopBit){
	portstream_fd pstream;
	DCB dcb;
	COMMTIMEOUTS timeout_info;

	pstream = 
		CreateFile(portname,
		           GENERIC_READ, // | GENERIC_WRITE,
				   0,    /* comm devices must be opened w/exclusive-access */
				   NULL, /* no security attrs */
				   OPEN_EXISTING, /* comm devices must use OPEN_EXISTING */
				   FILE_ATTRIBUTE_NORMAL, // | FILE_FLAG_OVERLAPPED,  // overlapped I/O
				   NULL  /* hTemplate must be NULL for comm devices */
				   );
	if (pstream == INVALID_HANDLE_VALUE) {
    	printf("\nCreateFile error for %s (error=%d)\n", portname, GetLastError());
      	return NULL;
		}

	if ( !(GetCommState(pstream, &dcb)) ) 
	   { printf("GetCommState error(%d)\n", GetLastError());
	     return NULL;
	   }

	// Setting COM port parameters
    // e.g. "COM1:9600,n,8,1"

	dcb.BaudRate = Speed;
    dcb.ByteSize = (unsigned char) Bits;
	//dcb.Parity = NOPARITY;
	if (StopBit == 2)
	     dcb.StopBits = TWOSTOPBITS; 
    else dcb.StopBits = ONESTOPBIT;
	switch (Parity) {
		case NO_PARITY:   dcb.Parity = NOPARITY;    break;
		case EVEN_PARITY: dcb.Parity = EVENPARITY;  break;
		case ODD_PARITY:  dcb.Parity = ODDPARITY;   break;
		default:		  return NULL;     }
	dcb.fOutxCtsFlow      = TRUE;
	dcb.fOutxDsrFlow      = FALSE;
	dcb.fDsrSensitivity   = FALSE;
	dcb.fOutX			  = FALSE;
	dcb.fInX			  = FALSE;
	dcb.fDtrControl       = DTR_CONTROL_ENABLE;
	dcb.fRtsControl       = RTS_CONTROL_ENABLE;
	dcb.fTXContinueOnXoff = FALSE;
	dcb.fBinary			  = TRUE;
	dcb.fParity			  = FALSE;



    if ( ! (SetCommState(pstream, &dcb)) ) 
	   { printf("SetCommState err(%d)\n", GetLastError());
	     return NULL;
	}
	// SetCommState OK

    if ( ! (PurgeComm(pstream, PURGE_TXABORT | PURGE_RXABORT |
								  PURGE_TXCLEAR | PURGE_RXCLEAR)) )
	   { printf("PurgeComm err(%d)\n", GetLastError());
	     return NULL; }

	// set timout read info. Readfile returns immediately, even if not enough data is avail
	if ( ! GetCommTimeouts(pstream, &timeout_info) )
	   { printf("GetCommTimeouts err(%d)\n", GetLastError());
	     return NULL; }
	timeout_info.ReadIntervalTimeout = MAXDWORD;
	timeout_info.ReadTotalTimeoutMultiplier =0;  // Ojo!! aqui no estaba este cero!
    timeout_info.ReadTotalTimeoutConstant   = 0;
	if (! SetCommTimeouts(pstream, &timeout_info) )
	   { printf("SetCommTimeouts err(%d)\n", GetLastError());
	     return NULL; }

	// printf("\nPort initialized!\n");
    // SerialStringOut(pstream, "\nSetSerial");

    return (pstream);
}


/*JF: Funcion para realizar la lectura de un solo caracter de puerto serial*/

char serialbytesin (portstream_fd portstream, unsigned char *buffer, unsigned int charCnt, long timeout) {
   COMSTAT COMstatus;
   UINT NumCharsAvail;
   DWORD ErrCode;
   unsigned long BytesRead;
   time_t start_time, end_time;
   int timeout_in_secs;

    if ( timeout != -1 ) 
	{
		time(&start_time);
		/*JF2: */
	    timeout_in_secs = (timeout+500) / 1000; // pone en un entero los milisegundos del timeout en segundos
	} // fin del primer if

	if (portstream == 0) 
	{      
		printf("\nERROR(SerialBytesIn): portstream not properly opened\n");
	    return(FALSE);
	}
	else 
	{ /* lectura de todos los datos,uno */
		for (; charCnt > 0 ;) 
		{	
			err = ClearCommError(portstream, &ErrCode, &COMstatus);
			if (err == FALSE) 
			{
				printf("\nERROR!(SerialBytesIn): ClearCommError error (%d)\n", GetLastError() );
			    return(FALSE);
			}
			NumCharsAvail = COMstatus.cbInQue;
			//num_char_disp2=NumCharsAvail;
			//num_char_disp=num_char_disp+NumCharsAvail;
			//printf("Num chars dispon: %d\n",NumCharsAvail);
			//if (peeked_char_avail)	NumCharsAvail++;
			if ( NumCharsAvail >= charCnt ) 
			{
			/*	if (peeked_char_avail) 
				{ 
					*buffer = peeked_char;
					peeked_char_avail = FALSE;
					buffer++;
					charCnt--;
				}*/
				//printf("Numero de Bytes disponibles %d ",NumCharsAvail);
        		err = ReadFile(portstream, (LPSTR) buffer, charCnt, &BytesRead, NULL);
				if (err == TRUE)
					charCnt--;
				else 
				{
					printf("\nERROR!!(SerialBytesIn): ReadFile error (%d)\n", GetLastError() );
					return((char) err);
				}
			}
		 	else 
			{
				if ( timeout != -1 ) 
				{
					time(&end_time);
				    if ( (end_time - start_time) > timeout_in_secs )
					return TIMEOUT_CHAR_READ;
				    //timeout -= 100;
			        //if (timeout <= 0) 
					// return TIMEOUT_CHAR_READ;  
				}
   				//Sleep(0);  /* give up the remaining timeslice */
			}

		// otherwise, keep looping until needed chars are available
		} // jf2: parentesis del for
	}     // jf2: parentesis del primer else
	return TRUE;
}


/*************Rutina para la escritura al puerto seria**********************************/
/* returns TRUE if all is OK */

char SerialBytesOut(portstream_fd portstream, unsigned char *buffer, int charCnt)
{   
	int BytesWritten;

	if (portstream == NULL) return FALSE;
	if ( WriteFile(portstream, (LPSTR) buffer, charCnt, (unsigned long *) &BytesWritten, NULL) != TRUE )
	   { printf("\nWriteFile error of %d\n", GetLastError());
		 return -1;  
	   }
	return TRUE;
}
