/*
ActivMedia Robotics Interface for Applications (ARIA)
Copyright (C) 2002, ActivMedia Robotics, LLC


     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

If you wish to redistribute ARIA under different terms, contact 
ActivMedia Robotics for information about a commercial version of ARIA at 
robots@activmedia.com or 
ActivMedia Robotics, 44 Concord Street, Peterborough, NH 03458; 603-924-9100

*/

#define uselaser 1
#define usesonar 1

//#include "Aria.h"
#include "process.h"
#include <math.h>
#include "rs4.h"
#include <mmsystem.h>

/*
  This is an example of how to use the limiting behaviors.
  
  The way it works is that it has a limiting behavior higher priority
  than the joydrive action behavior.  So the joydrive action can try
  to do whatever it wants, but it won't work.
*/

struct passed {
	ArRobot * probot;
	ArUtil * putil;
	ArTime * ptime;
};

portstream_fd _port;
extern float contour[133];

void Guarda(void*);

int main(void)
{
  // the serial connection (robot)
  ArSerialConnection serConn;
  // tcp connection (sim)
  ArTcpConnection tcpConn;
  // robot
  ArRobot robot;
  // utilities to sleep my thread
  ArUtil util;
  // para medir el tiempo
  ArTime mtime;

  passed pass;
  pass.probot = &robot;
  pass.putil = &util;
  pass.ptime = &mtime;
  
  // limiter for close obstacles
  ArActionLimiterForwards limiter("speed limiter near", 300, 600, 250);
  // limiter for far away obstacles
  ArActionLimiterForwards limiterFar("speed limiter far", 300, 1100, 400);
  // limiter so we don't bump things backwards
  // limiter for far away obstacles
  ArActionLimiterTableSensor tableLimiter;
  // limiter so we don't bump things backwards
  ArActionLimiterBackwards backwardsLimiter;
  // the joydrive action
  ArActionJoydrive joydriveAct;
  // the keydrive action
  ArActionKeydrive keydriveAct;
  
  // sonar device, so the limiter will work, this must be added to the robot
  ArSonarDevice sonar;

  
  
  //Para obtener la posicin del robot
  ArPose pose;


  // mandatory init
  Aria::init();

  
 

  printf("This program will allow you to use a joystick or keyboard to control the robot.\nYou can use the arrow keys to drive, and the spacebar to stop.\nFor joystick control press the trigger button and then drive.\nPress escape to exit.\n");

  // if we don't have a joystick, let 'em know
  if (!joydriveAct.joystickInited())
    printf("Do not have a joystick, only the arrow keys on the keyboard will work.\n");
  
  // set the joystick so it won't do anything if the button isn't pressed
  joydriveAct.setStopIfNoButtonPressed(false);

  // add the sonar to the robot
  robot.addRangeDevice(&sonar);
  
  // First we see if we can open the tcp connection, if we can we'll
  // assume we're connecting to the sim, and just go on...  if we
  // can't open the tcp it means the sim isn't there, so just try the
  // robot

  // modify this next line if you're not using default tcp connection
  tcpConn.setPort();
  //tcpConn.setPort("147.83.76.100",8101);

  // see if we can get to the simulator  (true is success)
  if (tcpConn.openSimple())
  //if (tcpConn.open("147.83.76.100",8101))
  {
    // we could get to the sim, so set the robots device connection to the sim
    printf("Connecting to simulator through tcp.\n");
    robot.setDeviceConnection(&tcpConn);
  }
  else
  {
    // we couldn't get to the sim, so set the port on the serial
    // connection and then set the serial connection as the robots
    // device

    // modify the next line if you're not using the first serial port
    // to talk to your robot
    serConn.setPort("COM2");
    printf("Could not connect to simulator, connecting to robot through serial.\n");
    robot.setDeviceConnection(&serConn);
  }

  // try to connect, if we fail exit
  if (!robot.blockingConnect())
  {
    printf("Could not connect to robot... exiting\n");
    Aria::shutdown();
    return 1;
  }

 

  // set the robots maximum velocity (sonar don't work at all well if you're
  // going faster)
  robot.setMaxTransVel(50); //400);
  robot.setMaxRotVel(5);

  // enable the motors, disable amigobot sounds
  robot.comInt(ArCommands::ENABLE, 1);
  robot.comInt(ArCommands::SOUNDTOG, 0);


  // add the cations, put the limiters on top, then have the action,
  // this will keep the action from being able to drive too fast and hit
  // something
  //robot.addAction(&tableLimiter, 100);
  //robot.addAction(&limiter, 95);
  //robot.addAction(&limiterFar, 90);
  //robot.addAction(&backwardsLimiter, 85);
  //robot.addAction(&joydriveAct, 50);
  robot.addAction(&keydriveAct, 45);

  // set the joydrive action so it'll let the keydrive action fire if
  // there is no button pressed
  joydriveAct.setStopIfNoButtonPressed(false);


   //Empieza el thread
    _beginthread(Guarda,0,(void *)&pass);

  // run the robot, true here so that the run will exit if connection lost
  robot.run(true);

  

  
  
  // now exit
  Aria::shutdown();
  return 0;
}


void Guarda(void *mypassed) {
	double x,y,thrad,laserx,lasery,velt,velr,sonarx[16],sonary[16];
	bool nofin, running;
	long mytime;
	ArSensorReading * psonarreading;
	char filename[80];
	int i;

	CTime theTime = CTime::GetCurrentTime();


  //Para guardar en el archivo
	FILE *fp;

	ArRobot * miprobot;
	ArUtil * miutil;
	ArTime * miptime;
	passed * ppass;
	ppass = (passed *)mypassed;
	miprobot = (ArRobot *)ppass->probot;
	miutil = (ArUtil *)ppass->putil;
	miptime = (ArTime *)ppass->ptime;



	// pon el timer en 0
	miptime->setToNow();

	//abre el archivo
	sprintf(filename,"irimap_%ld.log",theTime.GetTime());
	fp=fopen(filename,"w");
	nofin=true;

	// Nos aseguramos que el robot este funcionando para no escribir basura
	running = false; 
	while (!running) {
		running = miprobot->isRunning; 
		miutil->sleep(100);
	}

	// abre puerto para el laser
	if(uselaser)
		_port = openserial("COM1");
	
	while (nofin) {
		//obtiene la posicin del robot de acuerdo a los encoders
		
		//guarda en un archivo la posicin del robot en m y rad
		x = (miprobot->getX())/1000.0;
		y = (miprobot->getY())/1000.0;
		thrad=(miprobot->getTh())*PI/180.0;
		velt=(miprobot->getVel())/1000.0;
		velr=(miprobot->getRotVel())*PI/180.0;
		mytime = miptime->mSecSince();
		printf("position: %ld %lf, %lf, %lf %lf %lf\n",mytime,x,y,thrad,velt,velr);
		fprintf(fp,"position: %ld %lf, %lf, %lf %lf %lf\n",mytime,x,y,thrad,velt,velr);

		// lee del sonar
		if(usesonar) {
			for(i=0;i<16;i++) {
				psonarreading = miprobot->getSonarReading(i);
				sonarx[i] = psonarreading->getX();
				sonary[i] = psonarreading->getY();
			}
			mytime = miptime->mSecSince();
			fprintf(fp,"sonar: %ld ",mytime);
			printf("sonar: OK\n");
			for(i=0;i<16;i++)			
				fprintf(fp,"%f %f ",sonarx[i], sonary[i]);			
			fprintf(fp,"\n");
		}

		// lee del laser
		if(uselaser) {
			if(Mess(_port)) {
				mytime = miptime->mSecSince();
				fprintf(fp,"laser: %ld ",mytime);
				printf("laser: OK\n");
				for(int n=0;n<133;n++)	{
					//Como contour slo contiene distancias y nosotros lo que necesitamos son los
					//puntos debemos hacer la transformacin.Lo hacemos sabiendo que el lser escanea
					//190, empezando de -5 y acabando en 185, y hace DOTSPERSCAN lecturas.
					laserx=contour[n]*cos(n*190.0*4*PI/(528.0*180.0)-5*PI/180);
					lasery=contour[n]*sin(n*190.0*4*PI/(528.0*180.0)-5*PI/180)+0.15;
					// este 0.102 es la distancia que hay entre el laser y el centro 
					// de giro del robot. La medida no ha sido tomada sin mucha precisin
					//					printf("%f %f ", laserx, lasery);
					//					fprintf(fp,"%f %f ",laserx, lasery);
					// y si solo quiero imprimir los valores del laser hago esto
					fprintf(fp,"%f ",contour[n]);
				}
				fprintf(fp,"\n");
			}
			else {
					printf("laser: Mal, no he leido del laser\n");
			}
		}

		nofin=miprobot->isRunning;
		// miutil->sleep(100);
	}

	if(uselaser) closeserial(_port);
	fclose(fp);
	_endthread();
}



